require_relative "aspect"

class Job
  def run
    raise "subclass must override Command#execute"
  end
end

class PrintJob < Job
  def run
    puts "running PrintJob..."
  end
end

class BackupJob < Job
  def run
    puts "running BackupJob..."
  end
end

module Benchmark
  extend Aspect
  
  @@result = {}

  RUN = execution(class: /Job\z/, method: :run)

  around RUN do |join_point|
    t = Time.now
    begin
      join_point.proceed
    ensure
      ts = @@result[join_point.signature] ||= []
      ts.push(Time.now - t)
    end
  end

  def self.print_result
    puts "Benchmark result"
    printf("%20s %7s %15s\n", "signature", "count", "avg")
    @@result.each do |sig, ts|
      printf("%20s %7d %15f\n",
             sig, ts.count, ts.inject(&:+) / ts.count)
    end
  end
end

jobs = [PrintJob.new, BackupJob.new]
10.times do
  jobs.each(&:run)
end
Benchmark.print_result
